/*
 * (C) Copyright 2016 V-Key Solutions.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Contributors:
 * 		Rukmal Dias
 */

package vkey.android.cordova;

import java.io.File;
import java.util.ArrayList;
import java.util.List;
import android.net.Uri;


import org.apache.cordova.CallbackContext;
import org.apache.cordova.CordovaPlugin;
import org.apache.cordova.PluginResult;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import com.vkey.android.internal.vguard.engine.BasicThreatInfo;
import com.vkey.android.vguard.ActivityLifecycleHook;
import com.vkey.android.vguard.LocalBroadcastManager;
import com.vkey.android.vguard.VGExceptionHandler;
import com.vkey.android.vguard.VGuard;
import com.vkey.android.vguard.VGuardBroadcastReceiver;
import com.vkey.android.vguard.VGuardFactory;
import com.vkey.android.vguard.VGuardLifecycleHook;

import android.app.Activity;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.os.Environment;
import android.os.Parcelable;
import androidx.core.content.FileProvider;
import android.util.Log;

public class VGuardSetupPlugin implements VGExceptionHandler {
    
    private static volatile VGuardSetupPlugin sInstance;
    
    private CordovaPlugin mPlugin;
    
    private static final String PROFILE_LOADED_ACTION = "vkey.android.vguard.PROFILE_LOADED";
    private static final String VOS_READY_ACTION = "vkey.android.vguard.VOS_READY";
    private static final String VOS_FIRMWARE_RETURN_CODE_KEY = "vkey.android.vguard.FIRMWARE_RETURN_CODE";
    private static final String PROFILE_THREAT_RESPONSE = "vkey.android.vguard.PROFILE_THREAT_RESPONSE";
    
    private static final String CB_SCAN_COMPLETE = "cb_scan_complete";
    private static final String CB_VOS_READY = "cb_vos_ready";
    private static final String CB_VGUARD_INITIALIZE = "cb_vguard_initialize";
    private static final String CB_VGUARD_PAUSE = "cb_vguard_pause";
    private static final String CB_VGUARD_RESUME = "cb_vguard_resume";
    private static final String CB_VGUARD_ERROR = "cb_vguard_error";
    private static final String RET_MESSAGE_VGUARD_NOT_STARTED = "vguard is not started..";
    private static final String RET_MESSAGE_SUCCESS = "successful";
    
    private VGuard vGuardMgr;
    private Activity mActivity;

    // LifecycleHook to notify VGuard of activity's lifecycle
    private VGuardLifecycleHook hook;

    // For VGuard to notify host app of events
    private VGuardBroadcastReceiver broadcastRvcr;
    
    private CallbackContext mCallbackContext;
    
    private static final String TAG = VGuardSetupPlugin.class.getName();
    
    private VGuardSetupPlugin(CordovaPlugin plugin) {
        mPlugin = plugin;
    }
    
    public void setupVGuard(final CallbackContext callbackContext) {
        mCallbackContext = callbackContext;
        Activity activity = mPlugin.cordova.getActivity();
        
        broadcastRvcr = new VGuardBroadcastReceiver(activity) {
            @Override
            public void onReceive(Context context, Intent intent) {
                super.onReceive(context, intent);
                
                if (PROFILE_LOADED_ACTION.equals(intent.getAction())) {
                    JSONObject obj = getSuccessCbValues(CB_VGUARD_INITIALIZE, RET_MESSAGE_SUCCESS);
                    PluginResult result = new PluginResult(PluginResult.Status.OK, obj);
                    result.setKeepCallback(true);
                    mCallbackContext.sendPluginResult(result);
                    Log.d(TAG, "adb_test profile loaded..");
                }
                
                if (VOS_READY_ACTION.equals(intent.getAction())) {
                    long firmwareReturnCode = intent.getLongExtra(VOS_FIRMWARE_RETURN_CODE_KEY, 0);
                    // initialize secure file IO
                    //setupSecureFileIO();
                    
                    JSONObject obj = getSuccessCbValues(CB_VOS_READY, firmwareReturnCode + "");
                    PluginResult result = new PluginResult(PluginResult.Status.OK, obj);
                    result.setKeepCallback(true);
                    mCallbackContext.sendPluginResult(result);
                    Log.d(TAG, "adb_test vos ready..");
                }
                
                if (VGuardBroadcastReceiver.ACTION_SCAN_COMPLETE.equals(intent.getAction())) {
                    ArrayList<Parcelable> detectedThreats = (ArrayList<Parcelable>) intent.
                            getParcelableArrayListExtra(VGuardBroadcastReceiver.SCAN_COMPLETE_RESULT);
                    List<BasicThreatInfo> list = new ArrayList<BasicThreatInfo>();
                    
                    for (Parcelable info : detectedThreats) {
                        BasicThreatInfo threatinfo = (BasicThreatInfo) info;
                        list.add(threatinfo);
                    }
                    
                    JSONObject obj = getSuccessCbValues(CB_SCAN_COMPLETE, list);
                    PluginResult result = new PluginResult(PluginResult.Status.OK, obj);
                    result.setKeepCallback(true);
                    mCallbackContext.sendPluginResult(result);
                    Log.d(TAG, "adb_test scan completed..");
                }
                
                if (PROFILE_THREAT_RESPONSE.equals(intent.getAction())) {
//                    int profileThreatResponse = intent.getIntExtra(PROFILE_THREAT_RESPONSE, -1);
//                    if (profileThreatResponse == 0 || profileThreatResponse == 1) {
//                        mProfileThreatResponse = true;
//                    }
                    Log.d(TAG, "-- profile threat response..");
                }
            }
        };

        // register using LocalBroadcastManager only for keeping data within your app
        LocalBroadcastManager localBroadcastMgr = LocalBroadcastManager.getInstance(activity);

        // necessary for vguard to finish activity safely
        localBroadcastMgr.registerReceiver(broadcastRvcr, new IntentFilter(VGuardBroadcastReceiver.ACTION_FINISH));
        localBroadcastMgr.registerReceiver(broadcastRvcr, new IntentFilter(VGuardBroadcastReceiver.ACTION_SCAN_COMPLETE));
        localBroadcastMgr.registerReceiver(broadcastRvcr, new IntentFilter(PROFILE_LOADED_ACTION));
        localBroadcastMgr.registerReceiver(broadcastRvcr, new IntentFilter(VOS_READY_ACTION));
        localBroadcastMgr.registerReceiver(broadcastRvcr, new IntentFilter(PROFILE_THREAT_RESPONSE));

        // Setting up VGuard
        try {
            VGuardFactory.debug = false;
            
            vGuardMgr = new VGuardFactory().getVGuard(activity);
            vGuardMgr.setVGExceptionHandler(this);
            
            // necessary for VGuard to be informed of the activity's lifecycle
            hook = new ActivityLifecycleHook(vGuardMgr);
        } 
        catch (Exception e) {
            String msg = "A serious exception has occurred in V-Guard: (" + e.getMessage() + ")";
            if(mCallbackContext != null) {
                JSONObject obj = getSuccessCbValues(CB_VGUARD_ERROR, msg);
        		PluginResult result = new PluginResult(PluginResult.Status.ERROR, obj);
            	result.setKeepCallback(true);
            	mCallbackContext.sendPluginResult(result);
            }
            Log.e(TAG, msg);
        }
    }
    public void sendEmail(final String email, final CallbackContext callbackContext) {
            // Do something in response to button click
            Log.d("sendEmail", "Send Email in VGuard Setup Plugin");
            mActivity = mPlugin.cordova.getActivity();
            Intent emailIntent = new Intent(Intent.ACTION_SEND);
            emailIntent.putExtra(Intent.EXTRA_SUBJECT,"Troubleshooting Logs for Device");
            emailIntent.putExtra(Intent.EXTRA_EMAIL,new String[]{email});
            emailIntent.putExtra(Intent.EXTRA_TEXT,"Please find attached troubleshooting logs of App protection VGuard ");
            
            emailIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            emailIntent.addFlags(Intent.FLAG_GRANT_WRITE_URI_PERMISSION);
            String outputDbFilePath = Environment.getExternalStorageDirectory() + File.separator + "VGuard_Troubleshooting_Logs.zip";

            File logsZipFile = new File(outputDbFilePath);
            Log.d("send email",outputDbFilePath);


            if (logsZipFile.exists()) {
                logsZipFile.delete();
            }
            String[] inputFiles = new String[1];
            inputFiles[0] = mActivity.getDatabasePath("vguard").getAbsolutePath();

            Compress compress = new Compress(inputFiles, outputDbFilePath);
            compress.zip();

            logsZipFile = new File(outputDbFilePath);
            Uri content = FileProvider.getUriForFile(
                mActivity,
                mActivity.getApplicationContext().getPackageName() + ".provider", logsZipFile);
            // mailIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            

            try {
                if (logsZipFile.exists()) {
                    Log.d("send email","error");
                    // if(Build.VERSION.SDK_INT >= 23 ){
                    Log.d("send email","android 8 used ");
                    emailIntent.putExtra(Intent.EXTRA_STREAM,content);
                    // }
                    // emailIntent.putExtra(Intent.EXTRA_STREAM, Uri.fromFile(logsZipFile));
                }
            }
            catch(Exception e ){
                Log.d("send email",e.toString());
            }


            emailIntent.setType("text/plain");

            mActivity.startActivity(Intent.createChooser(emailIntent, "Send Mail"));
            mActivity.finish();

        }
    
    public void onPause(final CallbackContext callbackContext) {
        try {
            if(getIsVosStarted()) {
                vGuardMgr.onPause(hook);
                
                if(callbackContext != null) {
                    JSONObject obj = getSuccessCbValues(CB_VGUARD_PAUSE, RET_MESSAGE_SUCCESS);
                    PluginResult result = new PluginResult(PluginResult.Status.OK, obj);
                    result.setKeepCallback(true);
                    callbackContext.sendPluginResult(result);
                }
            }
            else {
                JSONObject cb = getSuccessCbValues(CB_VGUARD_ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
                PluginResult result = new PluginResult(PluginResult.Status.ERROR, cb);
                result.setKeepCallback(true);
                callbackContext.sendPluginResult(result);
            }
        }
        catch(Exception e) {
            if(callbackContext != null) {
                JSONObject obj = getSuccessCbValues(CB_VGUARD_ERROR, e.getMessage());
                PluginResult result = new PluginResult(PluginResult.Status.ERROR, obj);
                result.setKeepCallback(true);
                callbackContext.sendPluginResult(result);
            }
        }
    }
    
    public void onResume(final CallbackContext callbackContext) {
        try {
            if(getIsVosStarted()) {
                vGuardMgr.onResume(hook);
                
                if(callbackContext != null) {
                    JSONObject obj = getSuccessCbValues(CB_VGUARD_RESUME, RET_MESSAGE_SUCCESS);
                    PluginResult result = new PluginResult(PluginResult.Status.OK, obj);
                    result.setKeepCallback(true);
                    callbackContext.sendPluginResult(result);
                }
            }
            else {
                JSONObject cb = getSuccessCbValues(CB_VGUARD_ERROR, RET_MESSAGE_VGUARD_NOT_STARTED);
                PluginResult result = new PluginResult(PluginResult.Status.ERROR, cb);
                result.setKeepCallback(true);
                callbackContext.sendPluginResult(result);
            }
        }
        catch(Exception e) {
            if(callbackContext != null) {
                JSONObject obj = getSuccessCbValues(CB_VGUARD_ERROR, e.getMessage());
                PluginResult result = new PluginResult(PluginResult.Status.ERROR, obj);
                result.setKeepCallback(true);
                callbackContext.sendPluginResult(result);
            }
        }
    }
    
    private JSONObject getSuccessCbValues(String action, String value) {
        JSONObject obj = new JSONObject();
        try {
            obj.put("action", action);
            obj.put("values", value);
        } 
        catch (JSONException e) {
            Log.w(TAG, "-- getSuccessCbValues, " + e.getMessage());
        }
        return obj;
    }
    
    private JSONObject getSuccessCbValues(String action, List<BasicThreatInfo> values) {
        JSONObject obj = new JSONObject();
        JSONArray arr = new JSONArray();
        
        if(values != null && values.size()>0) {
            for (BasicThreatInfo ti : values) {
                if(ti != null) {
                    JSONObject js = new JSONObject();
                    try {
                        js.put("threatClass", ti.getThreatClass());
                        js.put("threatName", ti.getThreatName());
                        js.put("threatInfo", ti.getThreatInfo());
                        arr.put(js);
                    } 
                    catch (JSONException e) {
                    }
                }
            }
        }
        
        try {
            obj.put("action", action);
            obj.put("values", arr);
        } 
        catch (JSONException e) {
            Log.w(TAG, "-- getSuccessCbValues, " + e.getMessage());
        }
        return obj;
    }
    
    @Override
    public void handleException(Exception e) {
        String msg = "An exception has occurred in V-Guard: (" + e.getMessage() + ")";
        
        if(mCallbackContext != null) {
            JSONObject obj = getSuccessCbValues(CB_VGUARD_ERROR, msg);
            PluginResult result = new PluginResult(PluginResult.Status.ERROR, obj);
            result.setKeepCallback(true);
            mCallbackContext.sendPluginResult(result);
        }
        Log.e(TAG, msg);
    }
    

    public boolean getIsVosStarted() {
        boolean result = false;
        if(vGuardMgr != null) {
            result = vGuardMgr.getIsVosStarted();
        }
        return result;
    }
    
    public static VGuardSetupPlugin getInstance(CordovaPlugin plugin) {
        if(sInstance == null) {
            synchronized (VGuardSetupPlugin.class) {
                if(sInstance == null) {
                    sInstance = new VGuardSetupPlugin(plugin);
                }
            }
        }
        return sInstance;
    }
    
}
